@@ -19,32 +19,13 @@
 UniprocessFuture <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, globals = TRUE, packages = NULL, lazy = FALSE, local = TRUE, ...) {
   if (substitute) expr <- substitute(expr)
 
-  if (lazy) {
-    ## Evaluate in a local environment?
-    if (local) {
-      envir <- new.env(parent = envir)
-    } else {
-      if (!is.logical(globals) || globals) {
-        stop("Non-supported use of lazy uniprocess futures: Whenever argument 'local' is FALSE, then argument 'globals' must also be FALSE. Lazy uniprocess future evaluation in the calling environment (local = FALSE) can only be done if global objects are resolved at the same time.")
-      }
-    }
+  if (lazy && !local && (!is.logical(globals) || globals)) {
+    stop("Non-supported use of lazy uniprocess futures: Whenever argument 'local' is FALSE, then argument 'globals' must also be FALSE. Lazy uniprocess future evaluation in the calling environment (local = FALSE) can only be done if global objects are resolved at the same time.")
   }
 
-  ## Global objects
-  assignToTarget <- (is.list(globals) || inherits(globals, "Globals"))
+  ## Global objects?
   gp <- getGlobalsAndPackages(expr, envir = envir, tweak = tweakExpression, globals = globals)
-
-  ## Assign globals to "target" environment?
-  if (length(gp$globals) > 0 && (lazy || assignToTarget)) {
-    target <- new.env(parent = envir)
-    target[["...future_has_globals"]] <- TRUE
-    globalsT <- gp$globals
-    for (name in names(globalsT)) {
-      target[[name]] <- globalsT[[name]]
-    }
-    globalsT <- NULL
-    envir <- target
-  }
+  globals <- gp$globals
 
   ## Record packages?
   if (length(packages) > 0 || (length(gp$packages) > 0 && lazy)) {
@@ -52,8 +33,9 @@ UniprocessFuture <- function(expr = NULL, envir = parent.frame(), substitute = F
   }
   
   gp <- NULL
-
-  f <- Future(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, asynchronous = FALSE, local = local, globals = NULL, packages = packages, ...)
+ 
+  f <- Future(expr = expr, envir = envir, substitute = FALSE, lazy = lazy, asynchronous = FALSE, local = local, globals = globals, packages = packages, version = "1.8", ...)
+  f$.callResult <- TRUE
   structure(f, class = c("UniprocessFuture", class(f)))
 }
 
@@ -65,7 +47,7 @@ run.UniprocessFuture <- function(future, ...) {
   if (future$state != 'created') {
     label <- future$label
     if (is.null(label)) label <- "<none>"
-    stop(sprintf("A future ('%s') can only be launched once.", label))
+    stop(FutureError(sprintf("A future ('%s') can only be launched once.", label), future = future))
   }
 
   ## Assert that the process that created the future is
@@ -75,32 +57,21 @@ run.UniprocessFuture <- function(future, ...) {
   expr <- getExpression(future)
   envir <- future$envir
 
+  ## Assign globals to separate "globals" enclosure environment?
+  globals <- future$globals
+  if (length(globals) > 0) {
+    if (future$local) {
+      envir <- new.env(parent = envir)
+    }
+    for (name in names(globals)) {
+      envir[[name]] <- globals[[name]]
+    }
+  }
+
   ## Run future
   future$state <- 'running'
-
-
-  ## WORKAROUND: tryCatch() does not record the traceback and
-  ## it is too late to infer it when the error has been caught.
-  ## Because of this with we use withCallingHandlers() to
-  ## capture errors and if they occur we record the call trace.
-  current <- sys.nframe()
-  tryCatch({
-    withCallingHandlers({
-      future$value <- eval(expr, envir = envir)
-      future$state <- 'finished'
-    }, error = function(ex) {
-      calls <- sys.calls()
-      ## Drop fluff added by withCallingHandlers()
-      calls <- calls[seq_len(length(calls)-2L)]
-      ## Drop fluff added by outer tryCatch()
-      calls <- calls[-seq_len(current+7L)]
-      ## Drop fluff added by outer local = TRUE
-      if (future$local) calls <- calls[-seq_len(6L)]
-      ex$traceback <- calls
-      future$value <- ex
-      future$state <- 'failed'
-    })
-  }, error = function(ex) {})
+  future$result <- eval(expr, envir = envir, enclos = baseenv())
+  future$state <- 'finished'
 
   if (debug) mdebug("%s started (and completed)", class(future)[1])
   
@@ -111,27 +82,36 @@ run.UniprocessFuture <- function(future, ...) {
 }
 
 
+#' @export
+result.UniprocessFuture <- function(future, ...) {
+  result <- future$result
+  if (inherits(result, "FutureResult")) return(result)
+  
+  if (future$state == "created") {
+    run(future)
+  }
+
+  result <- future$result
+  if (inherits(result, "FutureResult")) return(result)
+
+  label <- future$label
+  if (is.null(label)) label <- "<none>"
+  stop(FutureError(sprintf("Internal error: Unexpected result retrieved for %s future (%s): %s", class(future)[1], sQuote(label), sQuote(hexpr(future$expr))), future = future))
+}
+
 
 #' @export
 resolved.UniprocessFuture <- function(x, ...) {
   if (x$lazy) {
-    ## resolved() for lazy uniprocess futures must force value()
+    ## resolved() for lazy uniprocess futures must force result()
     ## such that the future gets resolved.  The reason for this
     ## is so that polling is always possible, e.g.
     ## while(!resolved(f)) Sys.sleep(5);
-    value(x, signal = FALSE)
+    result(x)
   }
   NextMethod("resolved")
 }
 
-globals.UniprocessFuture <- function(future, ...) {
-  envir <- future$envir
-  globals <- names(envir)
-  if (!"...future_has_globals" %in% globals) return(NULL)
-  globals <- setdiff(globals, "...future_has_globals")
-  mget(globals, envir = envir, inherits = FALSE)
-}
-
 
 #' @rdname UniprocessFuture-class
 #' @export
